@@ -200,7 +200,7 @@ sub _listURI {
     my($listType, $realm, $query, $user, $noConstraint, $facets, $start, $count) = @_;
     $query = undef if defined $query && $query eq "";
     if (!$noConstraint && !$this->{isAuth} &&
-	$this->admin()->configValue("constrainIdentity")) {
+	$this->admin()->conf("constrainIdentity")) {
 	$query = $this->_addQuery($query, 'identityId=="${identityId}"', $user);
     }
 
@@ -233,7 +233,7 @@ sub _torusURI {
     # If $realm is undefined, then nothing is appended
     if (defined $realm && !$this->{isAuth}) {
 	# Realm override only works on the primary Torus
-	my $torusRealm = $this->admin()->configValue("torusRealm");
+	my $torusRealm = $this->admin()->conf("torusRealm");
 	$realm = $torusRealm if defined $torusRealm;
     }
 
@@ -416,7 +416,7 @@ sub add {
 
     $this->_check_unique($realm, $user, %data);
     my %extra = ();
-    if ($this->admin()->configValue("constrainIdentity")) {
+    if ($this->admin()->conf("constrainIdentity")) {
 	%extra = (identityId => $user->field("identityId"));
     }
 
@@ -540,9 +540,9 @@ sub _check_unique {
     my($realm, $user, %data) = @_;
 
     my @conds;
-    foreach my $ref ($this->admin()->config()->profile()->fields()) {
+    foreach my $ref ($this->admin()->configObject()->profile()->fields()) {
 	my($name) = @$ref;
-	if ($this->admin()->configValue("field.$name.unique")) {
+	if ($this->admin()->conf("field.$name.unique")) {
 	    my $val = $data{$name};
 	    push @conds, [ $name, $val ]
 		if defined $val && $val ne "";
@@ -550,6 +550,7 @@ sub _check_unique {
     }
 
     return if !@conds;
+    ### This is worryingly similar to _hash2cql()
     my $cql = join(" and ", map {
 	my($name, $val) = @$_;
 	$val =~ s/"/\\"/g;
@@ -567,11 +568,13 @@ sub _check_unique {
 sub _hash2cql {
     my(%args) = @_;
 
-    return join(" and ", map {
+    my $query = join(" and ", map {
 	my $val = $args{$_};
-	$val =~ s/"/\\"/g;
-	qq[$_="$val"];
+	$val =~ s/[""*?]/\\$&/g;
+	qq[$_=="$val"];
     } sort keys %args);
+    
+    return $query;
 }
 
 sub _http_fatal {
@@ -589,7 +592,7 @@ sub _makeXML {
     my $this = shift();
     my($type, $goneHash, %data) = @_;
 
-    $type = $this->admin()->configValue('type')
+    $type = $this->admin()->conf('type')
 	if !defined $type;
 
     my %allKeys = (%data, %$goneHash);
@@ -672,9 +675,9 @@ sub make_realms_for_record {
     # Add new realms as required
     my $i = 1;
     while (1) {
-	my $realm = $admin->configValue("realmOnNew.$i.url")
+	my $realm = $admin->conf("realmOnNew.$i.url")
 	    or last;
-	my $type = $admin->configValue("realmOnNew.$i.type")
+	my $type = $admin->conf("realmOnNew.$i.type")
 	    or die "realmOnNew.$i.type not specified";
 
 	my $uri = $this->_substitute($realm, \%data);
@@ -682,13 +685,13 @@ sub make_realms_for_record {
 
 	my $j = 1;
 	while (1) {
-	    my $pname = $admin->configValue("realmOnNew.$i.parent.$j.name")
+	    my $pname = $admin->conf("realmOnNew.$i.parent.$j.name")
 		or last;
-	    my $purl = $admin->configValue("realmOnNew.$i.parent.$j.url")
+	    my $purl = $admin->conf("realmOnNew.$i.parent.$j.url")
 		or die "realmOnNew.$i.parent.$j.url not specified";
-	    my $ppri = $admin->configValue("realmOnNew.$i.parent.$j.priority")
+	    my $ppri = $admin->conf("realmOnNew.$i.parent.$j.priority")
 		or die "realmOnNew.$i.parent.$j.priority not specified";
-	    my $prefresh = $admin->configValue("realmOnNew.$i.parent.$j.refreshAfter") || 0;
+	    my $prefresh = $admin->conf("realmOnNew.$i.parent.$j.refreshAfter") || 0;
 
 	    $this->_addParentByURI("${uri}parents/", $pname, $ppri, $prefresh, $purl);
 	    $j++;
@@ -709,7 +712,7 @@ sub remove_realms_for_record {
     # Remove realms as required
     my $i = 1;
     while (1) {
-	my $realm = $admin->configValue("realmOnNew.$i.url")
+	my $realm = $admin->conf("realmOnNew.$i.url")
 	    or last;
 
 	my $uri = $this->_substitute($realm, \%data);
@@ -1049,17 +1052,18 @@ sub _sortedList {
 
     my @facetNames;
     # The odd "categories:\054" tells the Torus to split the categories field on commas.
-    if ($this->admin()->configValue("showFacets")) {
-	my $facets = $this->admin()->configValue("facetList");
+    if ($this->admin()->conf("showFacets")) {
+	my $facets = $this->admin()->conf("facetList");
 	foreach my $facet (split(/\|/, $facets)) {
-	    ### Nasty hard-coded knowledge of specified fields' structures
-	    $facet .= ':\054' if $facet eq 'categories';
-	    $facet .= ':\040' if $facet eq 'capabilities';
+	    my $sep = $this->admin()->conf("facet.$facet.separator");
+	    if (defined $sep) {
+		$facet .= ':' . $sep;
+	    }
 	    push @facetNames, $facet;
 	}
     }
     push @facetNames, "displayName"
-	if $this->admin()->configValue("showTagCloud");
+	if $this->admin()->conf("showTagCloud");
     my $facetNames;
     $facetNames = join (",", @facetNames)
 	if @facetNames;
@@ -1085,7 +1089,7 @@ sub generateCloud {
 
     my $admin = $this->admin();
     return undef
-	if !$admin->configValue("showTagCloud");
+	if !$admin->conf("showTagCloud");
 
     require 'Masterkey/Admin/TagCloud.pm';
     my $words = $facets->{displayName}->{words};
@@ -1097,7 +1101,7 @@ sub generateCloud {
 	}
     }
 
-    $cloud->generate('uri' => ".", 'levels'=> $admin->configValue("tagCloudLevels"));
+    $cloud->generate('uri' => ".", 'levels'=> $admin->conf("tagCloudLevels"));
     return $cloud;
 }
 
